@@ -2,13 +2,14 @@
 
 import os
 from os.path import join, dirname, exists
-from typing import List
+from functools import partial
+import logging
 
 from gunicorn.config import KNOWN_SETTINGS
 from gunicorn.app.base import Application
 
-from .config import load_profile_config
-from .utils import walk_files
+from .config import load_profile_config, load_app_settings
+from .utils import walk_files, redirect_app_logger, redirect_logger
 from .env import get_project_name_from_env
 
 __all__ = ['GunicornApplication']
@@ -17,7 +18,7 @@ __all__ = ['GunicornApplication']
 class GunicornApplication(Application):
 
     def __init__(self):
-        self.options = self._make_options()
+        self.options: dict = None
         super().__init__()
 
     def set_option(self, key, value):
@@ -25,6 +26,7 @@ class GunicornApplication(Application):
             self.cfg.set(key, value)
 
     def load_config(self):
+        self.options = self._make_options()
         for key, value in self.options.items():
             if key in self.cfg.settings and value is not None:
                 self.cfg.set(key.lower(), value)
@@ -33,7 +35,15 @@ class GunicornApplication(Application):
         from guniflask.app import create_app
 
         self._set_default_env()
-        return create_app(get_project_name_from_env())
+        app_name = get_project_name_from_env()
+        app_settings = load_app_settings(app_name)
+
+        gunicorn_logger = logging.getLogger('gunicorn.error')
+        redirect_logger('guniflask', gunicorn_logger)
+        redirect_logger(app_name, gunicorn_logger)
+        app = create_app(app_name, settings=app_settings)
+        redirect_app_logger(app, gunicorn_logger)
+        return app
 
     def _make_options(self):
         pid_dir = os.environ['GUNIFLASK_PID_DIR']
@@ -46,7 +56,8 @@ class GunicornApplication(Application):
             'worker_class': 'gevent',
             'pidfile': join(pid_dir, '{}-{}.pid'.format(project_name, id_string)),
             'accesslog': join(log_dir, '{}-{}.access.log'.format(project_name, id_string)),
-            'errorlog': join(log_dir, '{}-{}.error.log'.format(project_name, id_string))
+            'errorlog': join(log_dir, '{}-{}.error.log'.format(project_name, id_string)),
+            'proc_name': project_name
         }
         options.update(self._make_profile_options(os.environ.get('GUNIFLASK_ACTIVE_PROFILES')))
         # if debug
@@ -54,12 +65,9 @@ class GunicornApplication(Application):
             options.update(self._make_debug_options())
         self._makedirs(options)
         # hook wrapper
-        self._set_hook_wrapper(options)
+        HookWrapper.wrap(options)
         return options
 
-    def _set_hook_wrapper(self, options):
-        HookWrapper.from_config(options)
-
     def _make_profile_options(self, active_profiles):
         conf_dir = os.environ['GUNIFLASK_CONF_DIR']
         gc = load_profile_config(conf_dir, 'gunicorn', profiles=active_profiles)
@@ -95,52 +103,40 @@ class GunicornApplication(Application):
 
     def _set_default_env(self):
         bind = self.options.get('bind', '127.0.0.1:8000')
-        if not isinstance(bind, List):
-            bind = [bind]
-        host = None
+        if not isinstance(bind, str):
+            raise ValueError('Invalid bind: {}'.format(bind))
+
         port = 80
-        for b in bind:
-            s = b.split(':')
-            if s[0] in ('unix', 'fd'):
-                continue
-            host = s[0]
-            if len(s) > 1:
-                try:
-                    port = int(s[1])
-                except ValueError:
-                    continue
-            break
-        os.environ['GUNIFLASK_BIND_HOST'] = host
-        os.environ['GUNIFLASK_BIND_PORT'] = str(port)
+        s = bind.split(':')
+        host = s[0]
+        if len(s) > 1:
+            port = int(s[1])
+
+        os.environ['GUNIFLASK_HOST'] = host
+        os.environ['GUNIFLASK_PORT'] = str(port)
 
 
 class HookWrapper:
     HOOKS = ['on_starting', 'on_reload', 'on_exit']
 
-    def __init__(self, config, on_starting=None, on_reload=None, on_exit=None):
-        self.config = config
-        self._on_starting = on_starting
-        self._on_reload = on_reload
-        self._on_exit = on_exit
+    def __init__(self, user_hooks, sys_hooks):
+        self.user_hooks = user_hooks
+        self.sys_hooks = sys_hooks
 
     @classmethod
-    def from_config(cls, config):
-        kw = {}
+    def wrap(cls, config, **kwargs):
+        user_hooks = {}
         for h in cls.HOOKS:
-            kw[h] = config.get(h)
-        wrapper = cls(config, **kw)
+            if h in config:
+                user_hooks[h] = config[h]
+        w = cls(user_hooks, kwargs)
         for h in cls.HOOKS:
-            config[h] = getattr(wrapper, h)
-        return wrapper
+            if h in w.user_hooks or h in w.sys_hooks:
+                config[h] = partial(w.on_event, key=h)
+        return w
 
-    def on_starting(self, server):
-        if self._on_starting is not None:
-            self._on_starting(server)
-
-    def on_reload(self, server):
-        if self._on_reload is not None:
-            self._on_reload(server)
-
-    def on_exit(self, server):
-        if self._on_exit is not None:
-            self._on_exit(server)
+    def on_event(self, server, key=None):
+        if key in self.user_hooks:
+            self.user_hooks[key](server)
+        if key in self.sys_hooks:
+            self.sys_hooks[key](server)
